//
//  MainViewController.swift
//  WRS-Sample-i
//
//  Created by Y.SONE on 2017/02/12.
//  Copyright © 2017年 AIMEX corporation. All rights reserved.
//

import UIKit
import CoreBluetooth
import AudioToolbox

class MainViewController: UIViewController, NORBluetoothManagerDelegate, ScannerDelegate {

    // MARK: - View Properties
    var bluetoothManager    : NORBluetoothManager?
    var uartPeripheralName  : String?
    var strBarData          : String = ""

    // MARK: - View Actions
    @IBAction func unwind(sender: UIStoryboardSegue) {
       
    }
    
    @IBAction func connectionButtonTapped(_ sender: Any) {
        bluetoothManager?.cancelPeripheralConnection()
    }
    
    @IBAction func clearButtonTapped(_ sender: Any) {
        self.dataWindow.text = ""
    }
    
    // MARK: - View OUtlets
    @IBOutlet weak var dataWindow: UITextView!
    @IBOutlet weak var connectionButton: UIButton!
    @IBOutlet weak var deviceTitle: UITextField! 
    @IBOutlet weak var deviceName: UITextField!
    
    // MARK: - UIViewController デリゲートメソッド
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // カーソル位置の自動調整機能を禁止
        self.automaticallyAdjustsScrollViewInsets = false
    }
   
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    // MARK: - Segue メソッド
    override func shouldPerformSegue(withIdentifier identifier: String, sender: Any?) -> Bool {
        // The 'scan' seque will be performed only if bluetoothManager == nil (if we are not connected already).
        return identifier != "scan" || self.bluetoothManager == nil
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        guard segue.identifier == "scan" else {
            return
        }
        
        // Set this contoller as scanner delegate
        let nc = segue.destination as! UINavigationController
        let controller = nc.childViewControllerForStatusBarHidden as! ScannerViewController
        // controller.filterUUID = CBUUID.init(string: NORServiceIdentifiers.uartServiceUUIDString)
        controller.delegate = self
    }

    // MARK: - ScannerView デリゲートメソッド
    func centralManagerDidSelectPeripheral(withManager aManager: CBCentralManager, andPeripheral aPeripheral: CBPeripheral) {
        bluetoothManager = NORBluetoothManager(withManager: aManager)
        bluetoothManager!.delegate = self
        
        if let name = aPeripheral.name {
            self.uartPeripheralName = name
            self.deviceName.text = name
        } else {
            self.uartPeripheralName = "device"
            self.deviceName.text = "No name"
        }
        self.connectionButton.setTitle("受信停止", for: UIControlState())
        bluetoothManager!.connectPeripheral(peripheral: aPeripheral)
    }

    // MARK: - BluetoothManager デリゲートメソッド
    func peripheralReady() {
        print("Peripheral is ready")
    }
    
    func peripheralNotSupported() {
        print("Peripheral is not supported")
    }
    
    func didConnectPeripheral(deviceName aName: String?) {
        // Scanner uses other queue to send events. We must edit UI in the main queue
        DispatchQueue.main.async(execute: {
            self.connectionButton.setTitle("受信停止", for: UIControlState())
        })
        
        //Following if condition display user permission alert for background notification
        if UIApplication.instancesRespond(to: #selector(UIApplication.registerUserNotificationSettings(_:))){
            UIApplication.shared.registerUserNotificationSettings(UIUserNotificationSettings(types: [.sound, .alert], categories: nil))
        }

        NotificationCenter.default.addObserver(self, selector: #selector(self.applicationDidEnterBackgroundCallback), name: NSNotification.Name.UIApplicationDidEnterBackground, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(self.applicationDidBecomeActiveCallback), name: NSNotification.Name.UIApplicationDidBecomeActive, object: nil)
    }

    func didDisconnectPeripheral() {
            // Scanner uses other queue to send events. We must edit UI in the main queue
            DispatchQueue.main.async(execute: {
                self.connectionButton.setTitle("受信開始", for: UIControlState())
                self.deviceName.text = "未接続"
                
                if NORAppUtilities.isApplicationInactive() {
                    NORAppUtilities.showBackgroundNotification(message: "Peripheral \(self.uartPeripheralName!) is disconnected")
                }

                self.uartPeripheralName = nil
            })
        NotificationCenter.default.removeObserver(self, name: NSNotification.Name.UIApplicationDidBecomeActive, object: nil)
        NotificationCenter.default.removeObserver(self, name: NSNotification.Name.UIApplicationDidEnterBackground, object: nil)
        bluetoothManager = nil
    }

    // UART Notificationデータ受信
    func didReceivedData(rxData aData: String) {
      
        // 分割パケットの総数/分割パケットインデックス値
        let packetCnt: UInt8 = Array(String(describing: aData).utf8)[0]
        let packetIdx: UInt8 = Array(String(describing: aData).utf8)[1]

        // 読み取りデータ部(先頭2byte以降)を抽出
        let text: String = aData.substring(from: aData.index(aData.startIndex, offsetBy: 2))
        self.strBarData += text

        // 最終パケットなら画面にデータを出力
        if (packetIdx == packetCnt) {

            DispatchQueue.main.async(execute: {
                // システムサウンド＆バイブレータ鳴動
                let soundIdRing:SystemSoundID = 1022    // Calypso
                AudioServicesPlaySystemSound(soundIdRing)

                // UITextViewへの表示テキスト追加
                self.dataWindow.text = self.dataWindow.text + self.strBarData + "\n"
            
                // 表示位置のオートスクロール
                let range = NSMakeRange(self.dataWindow.text.characters.count - 1, 0)
                self.dataWindow.scrollRangeToVisible(range)       

                // バーコードバッファのクリア
                self.strBarData = ""     
            })
         }
    }

    func applicationDidEnterBackgroundCallback(){
        NORAppUtilities.showBackgroundNotification(message: "You are still connected to \(self.uartPeripheralName!)")
    }
    
    func applicationDidBecomeActiveCallback(){
        UIApplication.shared.cancelAllLocalNotifications()
    }
    
}
